@@ -24,7 +24,9 @@
 #include "ephy-embed-shell.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-file-helpers.h"
+#include "ephy-history-service.h"
 #include "ephy-smaps.h"
+#include "ephy-snapshot-service.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-embed-private.h"
 
@@ -33,11 +35,11 @@
 #include <glib/gi18n.h>
 
 struct _EphyAboutHandlerPrivate {
-  char *style_sheet;
   EphySMaps *smaps;
 };
 
 #define EPHY_PAGE_TEMPLATE_OVERVIEW         "/org/gnome/epiphany/page-templates/overview.html"
+#define EPHY_PAGE_TEMPLATE_ABOUT_CSS        "ephy-resource:///org/gnome/epiphany/page-templates/about.css"
 
 G_DEFINE_TYPE (EphyAboutHandler, ephy_about_handler, G_TYPE_OBJECT)
 
@@ -46,7 +48,6 @@ ephy_about_handler_finalize (GObject *object)
 {
   EphyAboutHandler *handler = EPHY_ABOUT_HANDLER (object);
 
-  g_free (handler->priv->style_sheet);
   g_clear_object (&handler->priv->smaps);
 
   G_OBJECT_CLASS (ephy_about_handler_parent_class)->finalize (object);
@@ -67,23 +68,6 @@ ephy_about_handler_class_init (EphyAboutHandlerClass *klass)
   g_type_class_add_private (object_class, sizeof (EphyAboutHandlerPrivate));
 }
 
-static const char *
-ephy_about_handler_get_style_sheet (EphyAboutHandler *handler)
-{
-  if (!handler->priv->style_sheet) {
-    const gchar *file;
-    GError *error = NULL;
-
-    file = ephy_file ("about.css");
-    if (file && !g_file_get_contents (file, &handler->priv->style_sheet, NULL, &error)) {
-      g_debug ("%s", error->message);
-      g_error_free (error);
-    }
-  }
-
-  return handler->priv->style_sheet;
-}
-
 static EphySMaps *
 ephy_about_handler_get_smaps (EphyAboutHandler *handler)
 {
@@ -150,11 +134,11 @@ get_plugins_cb (WebKitWebContext *web_context,
   enabled = webkit_settings_get_enable_plugins (webkit_settings);
 
   data_str = g_string_new ("<html>");
-  g_string_append_printf (data_str, "<head><title>%s</title>"           \
-                          "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />" \
-                          "<style type=\"text/css\">%s</style></head><body>",
-                          _("Installed plugins"),
-                          ephy_about_handler_get_style_sheet (about_request->handler));
+  g_string_append_printf (data_str, "<head><title>%s</title>"
+                          "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
+                          "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS"\" rel=\"stylesheet\" type=\"text/css\">"
+                          "</head><body>",
+                          _("Installed plugins"));
   g_string_append_printf (data_str, "<h1>%s</h1>", _("Plugins"));
 
   if (!enabled)
@@ -229,11 +213,11 @@ handle_memory_finished_cb (EphyAboutHandler *handler,
 
   memory = g_task_propagate_pointer (G_TASK (result), NULL);
   if (memory) {
-    g_string_append_printf (data_str, "<head><title>%s</title>"         \
-                            "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />" \
-                            "<style type=\"text/css\">%s</style></head><body>",
-                            _("Memory usage"),
-                            ephy_about_handler_get_style_sheet (handler));
+    g_string_append_printf (data_str, "<head><title>%s</title>"
+                            "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
+                            "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS"\" rel=\"stylesheet\" type=\"text/css\">"
+                            "</head><body>",
+                            _("Memory usage"));
 
     g_string_append_printf (data_str, "<h1>%s</h1>", _("Memory usage"));
     g_string_append (data_str, memory);
@@ -281,8 +265,6 @@ ephy_about_handler_handle_about (EphyAboutHandler *handler,
 {
   char *data;
   char *version;
-  char *image_data = NULL;
-  const char *filename;
   GtkIconInfo *icon_info;
 
   version = g_strdup_printf (_("Version %s"), VERSION);
@@ -291,17 +273,13 @@ ephy_about_handler_handle_about (EphyAboutHandler *handler,
                                           "web-browser",
                                           256,
                                           GTK_ICON_LOOKUP_GENERIC_FALLBACK);
-  if (icon_info != NULL) {
-    filename = gtk_icon_info_get_filename (icon_info);
-    image_data = ephy_file_create_data_uri_for_filename (filename, NULL);
-  }
 
   data = g_strdup_printf ("<html><head><title>%s</title>"
                           "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
-                          "<style type=\"text/css\">%s</style></head>"  \
-                          "<body>"
+                          "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS"\" rel=\"stylesheet\" type=\"text/css\">"
+                          "</head><body>"
                           "<div class=\"dialog\">"
-                          "<img src=\"%s\"/>"
+                          "<img src=\"file://%s\"/>"
                           "<h1 id=\"about-title\">%s</h1>"
                           "<h2 id=\"about-subtitle\">%s</h2>"
                           "<p id=\"about-tagline\">“%s”</p>"
@@ -310,14 +288,14 @@ ephy_about_handler_handle_about (EphyAboutHandler *handler,
                           "</table>"
                           "</div></body></html>",
                           _("About Web"),
-                          ephy_about_handler_get_style_sheet (handler),
-                          image_data ? image_data : "",
+                          icon_info ? gtk_icon_info_get_filename (icon_info) : "",
                           _("Web"),
                           version,
                           _("A simple, clean, beautiful view of the web"),
                           "WebKit", webkit_get_major_version (), webkit_get_minor_version (), webkit_get_micro_version ());
   g_free (version);
-  g_free (image_data);
+  if (icon_info)
+    g_object_unref (icon_info);
 
   ephy_about_handler_finish_request (request, data, -1);
 
@@ -330,10 +308,10 @@ ephy_about_handler_handle_epiphany (EphyAboutHandler *handler,
 {
   char *data;
 
-  data = g_strdup_printf ("<html><head><title>%s</title>"
+  data = g_strdup_printf ("<html class=\"epiphany-html\"><head><title>%s</title>"
                           "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
-                          "<style type=\"text/css\">%s</style></head>"
-                          "<body class=\"epiphany-body\">"
+                          "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS"\" rel=\"stylesheet\" type=\"text/css\">"
+                          "</head><body class=\"epiphany-body\">"
                           "<div id=\"ephytext\">"
                           "“Il semble que la perfection soit atteinte non quand il n'y a plus rien à"
                           " ajouter, mais quand il n'y a plus rien à retrancher.”"
@@ -342,8 +320,7 @@ ephy_about_handler_handle_epiphany (EphyAboutHandler *handler,
                           "<!-- Terre des Hommes, III: L'Avion, p. 60 -->"
                           "Antoine de Saint-Exupéry"
                           "</div></body></html>",
-                          _("Web"),
-                          ephy_about_handler_get_style_sheet (handler));
+                          _("Web"));
 
   ephy_about_handler_finish_request (request, data, -1);
 
@@ -360,13 +337,12 @@ handle_applications_finished_cb (EphyAboutHandler *handler,
   GList *applications, *p;
 
   data_str = g_string_new (NULL);
-  g_string_append_printf (data_str, "<html><head><title>%s</title>" \
-                          "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />" \
-                          "<style type=\"text/css\">%s</style></head>"  \
-                          "<body class=\"applications-body\"><h1>%s</h1>" \
+  g_string_append_printf (data_str, "<html><head><title>%s</title>"
+                          "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
+                          "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS"\" rel=\"stylesheet\" type=\"text/css\">"
+                          "</head><body class=\"applications-body\"><h1>%s</h1>"
                           "<p>%s</p>",
                           _("Applications"),
-                          ephy_about_handler_get_style_sheet (handler),
                           _("Applications"),
                           _("List of installed web applications"));
 
@@ -374,24 +350,18 @@ handle_applications_finished_cb (EphyAboutHandler *handler,
 
   applications = g_task_propagate_pointer (G_TASK (result), NULL);
   for (p = applications; p; p = p->next) {
-    char *img_data = NULL, *img_data_base64 = NULL;
-    gsize data_length;
     EphyWebApplication *app = (EphyWebApplication*)p->data;
 
-    if (g_file_get_contents (app->icon_url, &img_data, &data_length, NULL))
-      img_data_base64 = g_base64_encode ((guchar*)img_data, data_length);
     g_string_append_printf (data_str,
                             "<form>" \
                             "<tbody><tr>" \
-                            "<td class=\"icon\"><img width=64 height=64 src=\"data:image/png;base64,%s\"></img></td>" \
+                            "<td class=\"icon\"><img width=64 height=64 src=\"file://%s\"></img></td>" \
                             "<td class=\"data\"><div class=\"appname\">%s</div><div class=\"appurl\">%s</div></td>" \
                             "<td class=\"input\"><input type=\"hidden\" name=\"app_id\" value=\"%s\"><input type=\"submit\" value=\"Delete\" id=\"%s\">" \
                             "</td><td class=\"date\">%s <br /> %s</td></tr></tbody></form>",
-                            img_data_base64, app->name, app->url, app->name, app->name,
+                            app->icon_url, app->name, app->url, app->name, app->name,
                             /* Note for translators: this refers to the installation date. */
                             _("Installed on:"), app->install_date);
-    g_free (img_data_base64);
-    g_free (img_data);
   }
 
   g_string_append (data_str, "</table></body></html>");
@@ -429,64 +399,78 @@ ephy_about_handler_handle_applications (EphyAboutHandler *handler,
   return TRUE;
 }
 
-static GString *
-ephy_about_handler_generate_overview_html (EphyOverviewStore *store)
+typedef struct {
+  char *url;
+  time_t mtime;
+} GetSnapshotPathAsyncData;
+
+static void
+got_snapshot_path_for_url_cb (EphySnapshotService *service,
+                              GAsyncResult *result,
+                              GetSnapshotPathAsyncData *data)
 {
-  GtkTreeIter iter;
-  GString *data_str;
-  char *row_url, *row_title;
-  char *row_snapshot;
-  char *thumbnail_style;
-  gboolean valid;
+  char *snapshot;
 
-  valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter);
-  data_str = g_string_new (NULL);
-
-  while (valid) {
-    gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
-      EPHY_OVERVIEW_STORE_URI, &row_url,
-      EPHY_OVERVIEW_STORE_TITLE, &row_title,
-      EPHY_OVERVIEW_STORE_SNAPSHOT_PATH, &row_snapshot,
-      -1);
-
-    thumbnail_style = row_snapshot ? g_strdup_printf (" style=\"background: url(file://%s) no-repeat;\"", row_snapshot) : NULL;
-    g_free (row_snapshot);
-
-    g_string_append_printf (data_str,
-      "<li>" \
-      "  <a class=\"overview-item\" title=\"%s\" href=\"%s\">" \
-      "    <div class=\"close-button\" onclick=\"removeFromOverview(this.parentNode,event)\" title=\"%s\">&#10006;</div>" \
-      "    <span class=\"thumbnail\"%s></span>" \
-      "    <span class=\"title\">%s</span>" \
-      "  </a>" \
-      "</li>",
-      g_markup_escape_text (row_title,-1), row_url, _("Remove from overview"),
-                            thumbnail_style ? thumbnail_style : "", row_title);
-
-    g_free (row_title);
-    g_free (row_url);
-    g_free (thumbnail_style);
-
-    valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter);
+  snapshot = ephy_snapshot_service_get_snapshot_path_for_url_finish (service, result, NULL);
+  if (snapshot) {
+    ephy_embed_shell_set_thumbanil_path (ephy_embed_shell_get_default (), data->url, data->mtime, snapshot);
+    g_free (snapshot);
   }
-  return data_str;
+  g_free (data->url);
+  g_free (data);
 }
 
-
-static gboolean
-ephy_about_handler_handle_html_overview (EphyAboutHandler *handler,
-                                        WebKitURISchemeRequest *request)
+static void
+history_service_query_urls_cb (EphyHistoryService *history,
+                               gboolean success,
+                               GList *urls,
+                               WebKitURISchemeRequest *request)
 {
   GString *data_str;
   GBytes *html_file;
   GString *html = g_string_new ("");
   gsize data_length;
-  EphyOverviewStore *store;
   char *lang;
+  GList *l;
 
-  store = EPHY_OVERVIEW_STORE (ephy_embed_shell_get_frecent_store (ephy_embed_shell_get_default ()));
+  data_str = g_string_new (NULL);
+
+  if (success) {
+    EphySnapshotService *snapshot_service = ephy_snapshot_service_get_default ();
+
+    for (l = urls; l; l = g_list_next (l)) {
+      EphyHistoryURL *url = (EphyHistoryURL *)l->data;
+      const char *snapshot;
+      char *thumbnail_style = NULL;
+
+      snapshot = ephy_snapshot_service_lookup_snapshot_path (snapshot_service, url->url);
+      if (!snapshot) {
+        GetSnapshotPathAsyncData *data = g_new (GetSnapshotPathAsyncData, 1);
+
+        data->url = g_strdup (url->url);
+        data->mtime = url->thumbnail_time;
+        ephy_snapshot_service_get_snapshot_path_for_url_async (ephy_snapshot_service_get_default (),
+                                                               url->url, url->thumbnail_time, NULL,
+                                                               (GAsyncReadyCallback)got_snapshot_path_for_url_cb,
+                                                               data);
+      } else {
+        thumbnail_style = g_strdup_printf (" style=\"background: url(file://%s) no-repeat;\"", snapshot);
+      }
+
+      g_string_append_printf (data_str,
+                              "<li>"
+                              "  <a class=\"overview-item\" title=\"%s\" href=\"%s\">"
+                              "    <div class=\"close-button\" onclick=\"removeFromOverview(this.parentNode,event)\" title=\"%s\">&#10006;</div>"
+                              "    <span class=\"thumbnail\"%s></span>"
+                              "    <span class=\"title\">%s</span>"
+                              "  </a>"
+                              "</li>",
+                              g_markup_escape_text (url->title, -1), url->url, _("Remove from overview"),
+                              thumbnail_style ? thumbnail_style : "", url->title);
+      g_free (thumbnail_style);
+    }
+  }
 
-  data_str = ephy_about_handler_generate_overview_html(store);
   html_file = g_resources_lookup_data (EPHY_PAGE_TEMPLATE_OVERVIEW, 0, NULL);
 
   lang = g_strdup (pango_language_to_string (gtk_get_default_language ()));
@@ -501,10 +485,29 @@ ephy_about_handler_handle_html_overview (EphyAboutHandler *handler,
 
   data_length = html->len;
   ephy_about_handler_finish_request (request, g_string_free (html, FALSE), data_length);
+  g_object_unref (request);
 
   g_string_free (data_str,TRUE);
   g_bytes_unref (html_file);
   g_free (lang);
+}
+
+static gboolean
+ephy_about_handler_handle_html_overview (EphyAboutHandler *handler,
+                                         WebKitURISchemeRequest *request)
+{
+  EphyHistoryService *history;
+  EphyHistoryQuery *query;
+
+  history = EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ()));
+  query = ephy_history_query_new ();
+  query->sort_type = EPHY_HISTORY_SORT_MOST_VISITED;
+  query->limit = EPHY_ABOUT_OVERVIEW_MAX_ITEMS;
+  query->ignore_hidden = TRUE;
+  ephy_history_service_query_urls (history, query, NULL,
+                                   (EphyHistoryJobCallback)history_service_query_urls_cb,
+                                   g_object_ref (request));
+  ephy_history_query_free (query);
 
   return TRUE;
 }
@@ -513,52 +516,35 @@ static gboolean
 ephy_about_handler_handle_incognito (EphyAboutHandler *handler,
                                      WebKitURISchemeRequest *request)
 {
-  GBytes *img_data;
-  char *img_data_base64 = NULL;
   char *data;
-  GError *error;
 
   if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) != EPHY_EMBED_SHELL_MODE_INCOGNITO)
     return FALSE;
 
-  img_data = g_resources_lookup_data ("/org/gnome/epiphany/incognito.png", 0, &error);
-  if (img_data != NULL) {
-    img_data_base64 = g_base64_encode ((guchar*)g_bytes_get_data (img_data, NULL),
-                                       g_bytes_get_size (img_data));
-  } else {
-    g_debug ("%s", error->message);
-    g_error_free (error);
-  }
-
-  data = g_strdup_printf ("<html>\n"                                   \
-                          "<div dir=\"%s\">\n"                         \
-                          "<head>\n"                                   \
-                          "<title>%s</title>\n"                        \
-                          "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />" \
-                          "<style type=\"text/css\">%s</style>\n"      \
-                          "</head>\n"                                  \
-                          "<body class=\"incognito-body\">\n"          \
-                          "  <div id=\"mainblock\">\n"                 \
-                          "    <div style=\"background: transparent url(data:image/png;base64,%s) no-repeat 10px center;\">\n" \
-                          "      <h1>%s</h1>\n"                        \
-                          "      <p>%s</p>\n"                          \
-                          "    </div>\n"                               \
-                          "  </div>\n"                                 \
-                          "</body>\n"                                  \
-                          "</div>\n"                                   \
+  data = g_strdup_printf ("<html>\n"
+                          "<div dir=\"%s\">\n"
+                          "<head>\n"
+                          "<title>%s</title>\n"
+                          "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
+                          "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS"\" rel=\"stylesheet\" type=\"text/css\">\n"
+                          "</head>\n"
+                          "<body class=\"incognito-body\">\n"
+                          "  <div id=\"mainblock\">\n"
+                          "    <div style=\"background: transparent url(ephy-resource:///org/gnome/epiphany/incognito.png) no-repeat 10px center;\">\n" \
+                          "      <h1>%s</h1>\n"
+                          "      <p>%s</p>\n"
+                          "    </div>\n"
+                          "  </div>\n"
+                          "</body>\n"
+                          "</div>\n"
                           "</html>\n",
                           gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL ? "rtl" : "ltr",
                           _("Private Browsing"),
-                          ephy_about_handler_get_style_sheet (handler),
-                          img_data_base64 ? img_data_base64 : "",
                           _("Private Browsing"),
                           _("You are currently browsing <em>incognito</em>. Pages viewed in this "
                             "mode will not show up in your browsing history and all stored "
                             "information will be cleared when you close the window."));
 
-  g_free (img_data_base64);
-  g_bytes_unref (img_data);
-
   ephy_about_handler_finish_request (request, data, -1);
 
   return TRUE;
