@@ -27,14 +27,15 @@
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-private.h"
 #include "ephy-embed-type-builtins.h"
+#include "ephy-embed-utils.h"
 #include "ephy-encodings.h"
 #include "ephy-file-helpers.h"
 #include "ephy-history-service.h"
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
 #include "ephy-snapshot-service.h"
-#include "ephy-web-extension.h"
 #include "ephy-web-extension-proxy.h"
+#include "ephy-web-extension-names.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -42,6 +43,7 @@
 
 #define PAGE_SETUP_FILENAME "page-setup-gtk.ini"
 #define PRINT_SETTINGS_FILENAME "print-settings.ini"
+#define OVERVIEW_RELOAD_DELAY 1000
 
 #define EPHY_EMBED_SHELL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED_SHELL, EphyEmbedShellPrivate))
 
@@ -52,8 +54,9 @@ struct _EphyEmbedShellPrivate
   GtkPageSetup *page_setup;
   GtkPrintSettings *print_settings;
   EphyEmbedShellMode mode;
-  EphyFrecentStore *frecent_store;
   EphyAboutHandler *about_handler;
+  guint update_overview_timeout_id;
+  guint hiding_overview_item;
   GDBusConnection *bus;
   GList *web_extensions;
   guint web_extensions_page_created_signal_id;
@@ -91,10 +94,14 @@ ephy_embed_shell_dispose (GObject *object)
 {
   EphyEmbedShellPrivate *priv = EPHY_EMBED_SHELL (object)->priv;
 
+  if (priv->update_overview_timeout_id > 0) {
+    g_source_remove (priv->update_overview_timeout_id);
+    priv->update_overview_timeout_id = 0;
+  }
+
   g_clear_object (&priv->encodings);
   g_clear_object (&priv->page_setup);
   g_clear_object (&priv->print_settings);
-  g_clear_object (&priv->frecent_store);
   g_clear_object (&priv->global_history_service);
   g_clear_object (&priv->about_handler);
 
@@ -161,6 +168,77 @@ web_extension_page_created (GDBusConnection *connection,
   g_signal_emit (shell, signals[PAGE_CREATED], 0, page_id, web_extension);
 }
 
+static void
+history_service_query_urls_cb (EphyHistoryService *service,
+                               gboolean success,
+                               GList *urls,
+                               EphyEmbedShell *shell)
+{
+  GList *l;
+
+  if (!success)
+    return;
+
+  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+    EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
+
+    ephy_web_extension_proxy_history_set_urls (web_extension, urls);
+  }
+}
+
+static void
+ephy_embed_shell_update_overview_urls (EphyEmbedShell *shell)
+{
+  EphyHistoryQuery *query;
+
+  query = ephy_history_query_new ();
+  query->sort_type = EPHY_HISTORY_SORT_MOST_VISITED;
+  query->limit = EPHY_ABOUT_OVERVIEW_MAX_ITEMS;
+  query->ignore_hidden = TRUE;
+
+  ephy_history_service_query_urls (shell->priv->global_history_service, query, NULL,
+                                   (EphyHistoryJobCallback) history_service_query_urls_cb,
+                                   shell);
+  ephy_history_query_free (query);
+}
+
+static void
+history_service_urls_visited_cb (EphyHistoryService *history,
+                                 EphyEmbedShell *shell)
+{
+   ephy_embed_shell_update_overview_urls (shell);
+}
+
+static gboolean
+ephy_embed_shell_update_overview_timeout_cb (EphyEmbedShell *shell)
+{
+  shell->priv->update_overview_timeout_id = 0;
+
+  if (shell->priv->hiding_overview_item > 0)
+    return FALSE;
+
+  ephy_embed_shell_update_overview_urls (shell);
+
+  return FALSE;
+}
+
+static void
+history_set_url_hidden_cb (EphyHistoryService *service,
+                           gboolean success,
+                           gpointer result_data,
+                           EphyEmbedShell *shell)
+{
+  shell->priv->hiding_overview_item--;
+
+  if (!success)
+    return;
+
+  if (shell->priv->update_overview_timeout_id > 0)
+    return;
+
+  ephy_embed_shell_update_overview_urls (shell);
+}
+
 static void
 web_extension_remove_from_overview (GDBusConnection *connection,
                                     const char *sender_name,
@@ -170,15 +248,22 @@ web_extension_remove_from_overview (GDBusConnection *connection,
                                     GVariant *parameters,
                                     EphyEmbedShell *shell)
 {
-  GtkTreeIter iter;
-  EphyFrecentStore *store;
   const char *url_to_remove;
 
   g_variant_get (parameters, "(&s)", &url_to_remove);
-  store = ephy_embed_shell_get_frecent_store (ephy_embed_shell_get_default ());
-  if (ephy_overview_store_find_url (EPHY_OVERVIEW_STORE(store), url_to_remove, &iter)) {
-	  ephy_frecent_store_set_hidden(store, &iter);
-  }
+
+  shell->priv->hiding_overview_item++;
+  ephy_history_service_set_url_hidden (shell->priv->global_history_service,
+                                       url_to_remove, TRUE, NULL,
+                                       (EphyHistoryJobCallback) history_set_url_hidden_cb,
+                                       shell);
+
+  if (shell->priv->update_overview_timeout_id > 0)
+    g_source_remove (shell->priv->update_overview_timeout_id);
+
+  /* Wait for the CSS animations to finish before refreshing */
+  shell->priv->update_overview_timeout_id =
+    g_timeout_add (OVERVIEW_RELOAD_DELAY, (GSourceFunc) ephy_embed_shell_update_overview_timeout_cb, shell);
 }
 
 static void
@@ -212,41 +297,6 @@ ephy_embed_shell_unwatch_web_extension (EphyWebExtensionProxy *web_extension,
   g_object_weak_unref (G_OBJECT (web_extension), (GWeakNotify)web_extension_destroyed, shell);
 }
 
-static void
-history_service_query_urls_cb (EphyHistoryService *service,
-                               gboolean success,
-                               GList *urls,
-                               EphyEmbedShell *shell)
-{
-  GList *l;
-
-  if (!success)
-    return;
-
-  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
-    EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
-
-    ephy_web_extension_proxy_history_set_urls (web_extension, urls);
-  }
-}
-
-static void
-history_service_urls_visited_cb (EphyHistoryService *history,
-                                 EphyEmbedShell *shell)
-{
-  EphyHistoryQuery *query;
-
-  query = ephy_history_query_new ();
-  query->sort_type = EPHY_HISTORY_SORT_MOST_VISITED;
-  query->limit = ephy_frecent_store_get_history_length (ephy_embed_shell_get_frecent_store (shell));
-  query->ignore_hidden = TRUE;
-
-  ephy_history_service_query_urls (history, query, NULL,
-                                   (EphyHistoryJobCallback) history_service_query_urls_cb,
-                                   shell);
-  ephy_history_query_free (query);
-}
-
 static void
 history_service_url_title_changed_cb (EphyHistoryService *service,
                                       const char *url,
@@ -308,14 +358,18 @@ history_service_cleared_cb (EphyHistoryService *service,
   }
 }
 
-static void
-frecent_store_snapshot_saved_cb (EphyOverviewStore *store,
-                                 const char *url,
-                                 const char *path,
-                                 EphyEmbedShell *shell)
+void
+ephy_embed_shell_set_thumbanil_path (EphyEmbedShell *shell,
+                                     const char *url,
+                                     time_t mtime,
+                                     const char *path)
 {
   GList *l;
 
+  ephy_history_service_set_url_thumbnail_time (shell->priv->global_history_service,
+                                               url, mtime,
+                                               NULL, NULL, NULL);
+
   for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
     EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
 
@@ -356,63 +410,11 @@ ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
     g_signal_connect (shell->priv->global_history_service, "cleared",
                       G_CALLBACK (history_service_cleared_cb),
                       shell);
-    g_signal_connect (ephy_embed_shell_get_frecent_store (shell), "snapshot-saved",
-                      G_CALLBACK (frecent_store_snapshot_saved_cb),
-                      shell);
   }
 
   return G_OBJECT (shell->priv->global_history_service);
 }
 
-static GdkPixbuf *
-ephy_embed_shell_get_overview_icon (const char *icon_name)
-{
-  GError *error = NULL;
-  GdkPixbuf *pixbuf;
-  const char *filename;
-
-  filename = ephy_file (icon_name);
-  pixbuf = gdk_pixbuf_new_from_file (filename, &error);
-
-  if (!pixbuf) {
-    g_warning ("Couldn't load icon: %s", error->message);
-    g_error_free (error);
-  }
-
-  return pixbuf;
-}
-
-/**
- * ephy_embed_shell_get_frecent_store:
- * @shell: a #EphyEmbedShell
- *
- * Gets the #EphyFrecentStore in the shell. This can be used
- * by EphyOverview implementors.
- *
- * Returns: (transfer none): a #EphyFrecentStore
- **/
-EphyFrecentStore *
-ephy_embed_shell_get_frecent_store (EphyEmbedShell *shell)
-{
-  GdkPixbuf *default_icon;
-
-  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
-
-  if (shell->priv->frecent_store == NULL) {
-    shell->priv->frecent_store = ephy_frecent_store_new ();
-    default_icon = ephy_embed_shell_get_overview_icon ("missing-thumbnail.png");
-    g_object_set (shell->priv->frecent_store,
-                  "history-service",
-                  ephy_embed_shell_get_global_history_service (shell),
-                  "history-length", 10,
-                  "default-icon", default_icon,
-                  NULL);
-    g_object_unref (default_icon);
-  }
-
-  return shell->priv->frecent_store;
-}
-
 /**
  * ephy_embed_shell_get_encodings:
  * @shell: the #EphyEmbedShell
@@ -449,6 +451,31 @@ about_request_cb (WebKitURISchemeRequest *request,
   ephy_about_handler_handle_request (shell->priv->about_handler, request);
 }
 
+static void
+ephy_resource_request_cb (WebKitURISchemeRequest *request)
+{
+  const char *path;
+  GInputStream *stream;
+  gsize size;
+  GError *error = NULL;
+
+  path = webkit_uri_scheme_request_get_path (request);
+  if (!g_resources_get_info (path, 0, &size, NULL, &error)) {
+    webkit_uri_scheme_request_finish_error (request, error);
+    g_error_free (error);
+    return;
+  }
+
+  stream = g_resources_open_stream (path, 0, &error);
+  if (stream) {
+    webkit_uri_scheme_request_finish (request, stream, size, NULL);
+    g_object_unref (stream);
+  } else {
+    webkit_uri_scheme_request_finish_error (request, error);
+    g_error_free (error);
+  }
+}
+
 static void
 initialize_web_extensions (WebKitWebContext* web_context,
                            EphyEmbedShell *shell)
@@ -574,6 +601,11 @@ ephy_embed_shell_startup (GApplication* application)
   webkit_security_manager_register_uri_scheme_as_local (webkit_web_context_get_security_manager (web_context),
                                                         EPHY_ABOUT_SCHEME);
 
+  /* ephy-resource handler */
+  webkit_web_context_register_uri_scheme (web_context, "ephy-resource",
+                                          (WebKitURISchemeRequestCallback)ephy_resource_request_cb,
+                                          NULL, NULL);
+
   /* Store cookies in moz-compatible SQLite format */
   cookie_manager = webkit_web_context_get_cookie_manager (web_context);
   filename = g_build_filename (ephy_dot_dir (), "cookies.sqlite", NULL);
@@ -585,8 +617,6 @@ ephy_embed_shell_startup (GApplication* application)
                                          EPHY_PREFS_WEB_COOKIES_POLICY);
   ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, cookie_policy);
   g_free (cookie_policy);
-
-  ephy_embed_prefs_init ();
 }
 
 static void
@@ -613,7 +643,8 @@ ephy_embed_shell_shutdown (GApplication* application)
 
   g_list_foreach (priv->web_extensions, (GFunc)ephy_embed_shell_unwatch_web_extension, application);
 
-  ephy_embed_prefs_shutdown ();
+  g_object_unref (ephy_embed_prefs_get_web_view_group ());
+  ephy_embed_utils_shutdown ();
 }
 
 static void
