@@ -25,12 +25,17 @@
 #include "ephy-embed-utils.h"
 
 #include "ephy-about-handler.h"
+#include "ephy-embed-private.h"
+#include "ephy-settings.h"
 #include "ephy-string.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
 #include <libsoup/soup.h>
 
+static GRegex *non_search_regex;
+static GRegex *domain_regex;
+
 char *
 ephy_embed_utils_link_message_parse (const char *message)
 {
@@ -70,6 +75,38 @@ ephy_embed_utils_link_message_parse (const char *message)
   return g_string_free (tmp, FALSE);
 }
 
+static gpointer
+create_non_search_regex (gpointer user_data)
+{
+  non_search_regex = g_regex_new (EPHY_WEB_VIEW_NON_SEARCH_REGEX,
+                                  G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
+  return non_search_regex;
+}
+
+static GRegex *
+get_non_search_regex (void)
+{
+  static GOnce once_init = G_ONCE_INIT;
+
+  return g_once (&once_init, create_non_search_regex, NULL);
+}
+
+static gpointer
+create_domain_regex (gpointer user_data)
+{
+  domain_regex = g_regex_new (EPHY_WEB_VIEW_DOMAIN_REGEX,
+                              G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
+  return domain_regex;
+}
+
+static GRegex *
+get_domain_regex (void)
+{
+  static GOnce once_init = G_ONCE_INIT;
+
+  return g_once (&once_init, create_domain_regex, NULL);
+}
+
 gboolean
 ephy_embed_utils_address_has_web_scheme (const char *address)
 {
@@ -104,48 +141,113 @@ ephy_embed_utils_address_is_existing_absolute_filename (const char *address)
     g_file_test (address, G_FILE_TEST_EXISTS);
 }
 
+static gboolean
+is_public_domain (const char *address)
+{
+  char *host;
+  gboolean retval = FALSE;
+
+  host = ephy_string_get_host_name (address);
+  if (!host)
+    return FALSE;
+
+  if (g_regex_match (get_domain_regex (), host, 0, NULL)) {
+    if (g_str_equal (host, "localhost"))
+      retval = TRUE;
+    else {
+      const char *end;
+
+      end = g_strrstr (host, ".");
+      if (end && *end != '\0')
+        retval = soup_tld_domain_is_public_suffix (end);
+    }
+  }
+
+  g_free (host);
+
+  return retval;
+}
+
+gboolean
+ephy_embed_utils_address_is_valid (const char *address)
+{
+  char *scheme;
+  gboolean retval;
+
+  if (!address)
+    return FALSE;
+
+  scheme = g_uri_parse_scheme (address);
+
+  retval = scheme ||
+    ephy_embed_utils_address_is_existing_absolute_filename (address) ||
+    g_regex_match (get_non_search_regex (), address, 0, NULL) ||
+    is_public_domain (address);
+
+  g_free (scheme);
+
+  return retval;
+}
+
 char*
 ephy_embed_utils_normalize_address (const char *address)
 {
   char *effective_address = NULL;
-  SoupURI *uri;
 
   g_return_val_if_fail (address, NULL);
 
   if (ephy_embed_utils_address_is_existing_absolute_filename (address))
     return g_strconcat ("file://", address, NULL);
 
-  uri = soup_uri_new (address);
+  if (g_str_has_prefix (address, "about:") && !g_str_equal (address, "about:blank"))
+    return g_strconcat (EPHY_ABOUT_SCHEME, address + strlen ("about"), NULL);
 
-  /* FIXME: if we are here we passed through the "should we
-   * auto-search this?" regex in EphyWebView, so we should be a
-   * valid-ish URL. Auto-prepend http:// to anything that is not
-   * one according to soup, because it probably will be
-   * something like "google.com". Special case localhost(:port)
-   * and IP(:port), because SoupURI, correctly, thinks it is a
-   * URI with scheme being localhost/IP and, optionally, path
-   * being the port. Ideally we should check if we have a
-   * handler for the scheme, and since we'll fail for localhost
-   * and IP, we'd fallback to loading it as a domain. */
-  if (!uri ||
-      (uri && !g_strcmp0 (uri->scheme, "localhost")) ||
-      (uri && g_hostname_is_ip_address (uri->scheme)))
-    effective_address = g_strconcat ("http://", address, NULL);
-  else {
-    /* Convert about: schemes to ephy-about: in order to
-     * force WebKit to delegate its handling to our
-     * EphyRequestAbout. In general about: schemes are
-     * handled internally by WebKit and mean "empty
-     * document".
-     */
-    if (g_str_has_prefix (address, "about:") && !g_str_equal (address, "about:blank")) {
-      effective_address = g_strconcat (EPHY_ABOUT_SCHEME, address + strlen ("about"), NULL);
-    } else
-      effective_address = g_strdup (address);
+  if (!ephy_embed_utils_address_has_web_scheme (address)) {
+    SoupURI *uri;
+
+    uri = soup_uri_new (address);
+
+    /* Auto-prepend http:// to anything that is not
+     * one according to soup, because it probably will be
+     * something like "google.com". Special case localhost(:port)
+     * and IP(:port), because SoupURI, correctly, thinks it is a
+     * URI with scheme being localhost/IP and, optionally, path
+     * being the port. Ideally we should check if we have a
+     * handler for the scheme, and since we'll fail for localhost
+     * and IP, we'd fallback to loading it as a domain. */
+    if (!uri ||
+        (uri && !g_strcmp0 (uri->scheme, "localhost")) ||
+        (uri && g_hostname_is_ip_address (uri->scheme)))
+      effective_address = g_strconcat ("http://", address, NULL);
+
+    if (uri)
+      soup_uri_free (uri);
   }
 
-  if (uri)
-    soup_uri_free (uri);
+  return effective_address ? effective_address : g_strdup (address);
+}
+
+char *
+ephy_embed_utils_normalize_or_autosearch_address (const char *address)
+{
+  char *query_param, *url_search;
+  char *effective_address;
+
+  if (ephy_embed_utils_address_is_valid (address))
+    return ephy_embed_utils_normalize_address (address);
+
+  url_search = g_settings_get_string (EPHY_SETTINGS_MAIN,
+                                      EPHY_PREFS_KEYWORD_SEARCH_URL);
+  if (url_search == NULL || url_search[0] == '\0') {
+    g_free (url_search);
+    url_search = g_strdup (_("http://duckduckgo.com/?q=%s&amp;t=epiphany"));
+  }
+
+  query_param = soup_form_encode ("q", address, NULL);
+  /* + 2 here is getting rid of 'q=' */
+  effective_address = g_strdup_printf (url_search, query_param + 2);
+  g_free (query_param);
+  g_free (url_search);
 
   return effective_address;
 }
@@ -186,3 +288,10 @@ ephy_embed_utils_is_no_show_address (const char *address)
 
   return FALSE;
 }
+
+void
+ephy_embed_utils_shutdown (void)
+{
+  g_clear_pointer (&non_search_regex, (GDestroyNotify)g_regex_unref);
+  g_clear_pointer (&domain_regex, (GDestroyNotify)g_regex_unref);
+}
