@@ -33,21 +33,19 @@
 G_BEGIN_DECLS
 
 #define EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED(embed) (WEBKIT_WEB_VIEW (ephy_embed_get_web_view (embed)))
-#ifdef HAVE_WEBKIT2
-/* There's no scrolled window widget in WebKit2 */
 #define EPHY_GET_EMBED_FROM_EPHY_WEB_VIEW(view) (EPHY_EMBED (gtk_widget_get_parent (gtk_widget_get_parent (gtk_widget_get_parent ((GTK_WIDGET (view)))))))
-#else
-#define EPHY_GET_EMBED_FROM_EPHY_WEB_VIEW(view) (EPHY_EMBED (gtk_widget_get_parent (gtk_widget_get_parent (gtk_widget_get_parent (gtk_widget_get_parent ((GTK_WIDGET (view))))))))
-#endif
 
 #define EPHY_WEBKIT_BACK_FORWARD_LIMIT 100
 
 char*    ephy_embed_utils_link_message_parse                    (const char *message);
 gboolean ephy_embed_utils_address_has_web_scheme                (const char *address);
 gboolean ephy_embed_utils_address_is_existing_absolute_filename (const char *address);
+gboolean ephy_embed_utils_address_is_valid                      (const char *address);
 char*    ephy_embed_utils_normalize_address                     (const char *address);
+char *   ephy_embed_utils_normalize_or_autosearch_address       (const char *address);
 gboolean ephy_embed_utils_url_is_empty                          (const char *location);
 gboolean ephy_embed_utils_is_no_show_address                    (const char *address);
+void     ephy_embed_utils_shutdown                              (void);
 
 G_END_DECLS
 
