@@ -39,6 +39,7 @@
 #include "ephy-history-service.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
+#include "ephy-snapshot-service.h"
 #include "ephy-string.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-web-dom-utils.h"
@@ -100,7 +101,7 @@ struct _EphyWebViewPrivate {
   EphyHistoryService *history_service;
   GCancellable *history_service_cancellable;
 
-  guint snapshot_idle_id;
+  guint snapshot_timeout_id;
 
   EphyHistoryPageVisitType visit_type;
 
@@ -581,6 +582,50 @@ ephy_web_view_history_cleared_cb (EphyHistoryService *history_service,
   ephy_web_view_clear_history (view);
 }
 
+typedef struct {
+  char *url;
+  time_t mtime;
+} GetSnapshotPathAsyncData;
+
+static void
+got_snapshot_path_cb (EphySnapshotService *service,
+                      GAsyncResult *result,
+                      GetSnapshotPathAsyncData *data)
+{
+  char *snapshot;
+
+  snapshot = ephy_snapshot_service_get_snapshot_path_finish (service, result, NULL);
+  if (snapshot) {
+    ephy_embed_shell_set_thumbanil_path (ephy_embed_shell_get_default (), data->url, data->mtime, snapshot);
+    g_free (snapshot);
+  }
+  g_free (data->url);
+  g_free (data);
+}
+
+static gboolean
+web_view_check_snapshot (WebKitWebView *web_view)
+{
+  EphyWebView* view = EPHY_WEB_VIEW (web_view);
+  EphySnapshotService *service = ephy_snapshot_service_get_default ();
+  const char *url = webkit_web_view_get_uri (web_view);
+  GetSnapshotPathAsyncData *data;
+
+  view->priv->snapshot_timeout_id = 0;
+
+  if (ephy_snapshot_service_lookup_snapshot_path (service, url))
+    return FALSE;
+
+  data = g_new (GetSnapshotPathAsyncData, 1);
+  data->url = g_strdup (url);
+  data->mtime = time (NULL);
+  ephy_snapshot_service_get_snapshot_path_async (service, web_view, data->mtime, NULL,
+                                                 (GAsyncReadyCallback)got_snapshot_path_cb,
+                                                 data);
+
+  return FALSE;
+}
+
 static void
 _ephy_web_view_update_icon (EphyWebView *view)
 {
@@ -605,6 +650,11 @@ icon_changed_cb (EphyWebView *view,
                  GParamSpec *pspec,
                  gpointer user_data)
 {
+  if (view->priv->snapshot_timeout_id == 0) {
+    view->priv->snapshot_timeout_id = g_timeout_add_full (G_PRIORITY_LOW, 0,
+                                                          (GSourceFunc)web_view_check_snapshot,
+                                                          view, NULL);
+  }
   _ephy_web_view_update_icon (view);
 }
 
@@ -694,9 +744,9 @@ ephy_web_view_dispose (GObject *object)
     g_clear_object (&priv->history_service_cancellable);
   }
 
-  if (priv->snapshot_idle_id) {
-    g_source_remove (priv->snapshot_idle_id);
-    priv->snapshot_idle_id = 0;
+  if (priv->snapshot_timeout_id) {
+    g_source_remove (priv->snapshot_timeout_id);
+    priv->snapshot_timeout_id = 0;
   }
 
   g_clear_object(&priv->certificate);
@@ -709,16 +759,6 @@ ephy_web_view_finalize (GObject *object)
 {
   EphyWebViewPrivate *priv = EPHY_WEB_VIEW (object)->priv;
 
-  if (priv->non_search_regex) {
-    g_regex_unref (priv->non_search_regex);
-    priv->non_search_regex = NULL;
-  }
-
-  if (priv->domain_regex) {
-    g_regex_unref (priv->domain_regex);
-    priv->domain_regex = NULL;
-  }
-
   ephy_web_view_popups_manager_reset (EPHY_WEB_VIEW (object));
 
   g_free (priv->address);
@@ -1514,58 +1554,6 @@ ephy_web_view_location_changed (EphyWebView *view,
   g_object_thaw_notify (object);
 }
 
-static void
-on_snapshot_ready (WebKitWebView *webview,
-                   GAsyncResult *res,
-                   GtkTreeRowReference *ref)
-{
-  GtkTreeModel *model;
-  cairo_surface_t *surface;
-  GError *error = NULL;
-
-  surface = webkit_web_view_get_snapshot_finish (webview, res, &error);
-  if (error) {
-    g_warning ("%s(): %s", G_STRFUNC, error->message);
-    g_error_free (error);
-    return;
-  }
-
-  model = gtk_tree_row_reference_get_model (ref);
-  ephy_overview_store_set_snapshot (EPHY_OVERVIEW_STORE (model), ref, surface,
-                                    webkit_web_view_get_favicon (webview));
-  cairo_surface_destroy (surface);
-}
-
-/* FIXME: We should be using the snapshot service for this instead of
-   using the WK API directly. */
-static gboolean
-web_view_check_snapshot (WebKitWebView *web_view)
-{
-  EphyOverviewStore *store;
-  GtkTreeIter iter;
-  GtkTreeRowReference *ref;
-  GtkTreePath *path;
-  EphyWebView* view = EPHY_WEB_VIEW (web_view);
-
-  EphyEmbedShell *embed_shell = ephy_embed_shell_get_default ();
-
-  store = EPHY_OVERVIEW_STORE (ephy_embed_shell_get_frecent_store (embed_shell));
-  if (ephy_overview_store_find_url (store, webkit_web_view_get_uri (web_view), &iter) &&
-      ephy_overview_store_needs_snapshot (store, &iter)) {
-    path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
-    ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (store), path);
-    gtk_tree_path_free (path);
-    webkit_web_view_get_snapshot (web_view, WEBKIT_SNAPSHOT_REGION_VISIBLE,
-                                  WEBKIT_SNAPSHOT_OPTIONS_NONE,
-                                  NULL, (GAsyncReadyCallback)on_snapshot_ready,
-                                  ref);
-  }
-
-  view->priv->snapshot_idle_id = 0;
-
-  return FALSE;
-}
-
 static void
 load_changed_cb (WebKitWebView *web_view,
                  WebKitLoadEvent load_event,
@@ -1583,6 +1571,11 @@ load_changed_cb (WebKitWebView *web_view,
 
     priv->load_failed = FALSE;
 
+    if (priv->snapshot_timeout_id) {
+      g_source_remove (priv->snapshot_timeout_id);
+      priv->snapshot_timeout_id = 0;
+    }
+
     loading_uri = webkit_web_view_get_uri (web_view);
     g_signal_emit_by_name (view, "new-document-now", loading_uri);
 
@@ -1660,10 +1653,18 @@ load_changed_cb (WebKitWebView *web_view,
     /* Reset visit type. */
     priv->visit_type = EPHY_PAGE_VISIT_NONE;
 
-    if (!ephy_web_view_is_history_frozen (view)) {
-      if (priv->snapshot_idle_id)
-        g_source_remove (priv->snapshot_idle_id);
-      priv->snapshot_idle_id = g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc)web_view_check_snapshot, web_view, NULL);
+    if (!ephy_web_view_is_history_frozen (view) &&
+        ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) != EPHY_EMBED_SHELL_MODE_INCOGNITO) {
+      if (!ephy_snapshot_service_lookup_snapshot_path (ephy_snapshot_service_get_default (), webkit_web_view_get_uri (web_view))) {
+        /* If the snapshot check hasn't been scheduled already by the favicon callback,
+         * check the snapshot if we don't get a favicon in 1 second
+         */
+        if (priv->snapshot_timeout_id == 0) {
+          priv->snapshot_timeout_id = g_timeout_add_seconds_full (G_PRIORITY_LOW, 1,
+                                                                  (GSourceFunc)web_view_check_snapshot,
+                                                                  web_view, NULL);
+        }
+      }
     }
 
     ephy_web_view_thaw_history (view);
@@ -1955,12 +1956,6 @@ ephy_web_view_init (EphyWebView *web_view)
 
   priv->file_monitor = ephy_file_monitor_new (web_view);
 
-  priv->non_search_regex = g_regex_new (EPHY_WEB_VIEW_NON_SEARCH_REGEX,
-                                        G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
-
-  priv->domain_regex = g_regex_new (EPHY_WEB_VIEW_DOMAIN_REGEX,
-                                    G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
-
   priv->history_service = EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ()));
   priv->history_service_cancellable = g_cancellable_new ();
 
@@ -2045,88 +2040,6 @@ ephy_web_view_new_with_related_view (WebKitWebView *related_view)
                        NULL);
 }
 
-static gboolean
-is_public_domain (EphyWebView *view, const char *url)
-{
-  gboolean retval = FALSE;
-  char *host = NULL;
-  EphyWebViewPrivate *priv = view->priv;
-
-  host = ephy_string_get_host_name (url);
-  g_return_val_if_fail (host, FALSE);
-
-  if (g_regex_match (priv->domain_regex, host, 0, NULL)) {
-    if (g_str_equal (host, "localhost"))
-      retval = TRUE;
-    else {
-      const char *end;
-
-      end = g_strrstr (host, ".");
-
-      if (end && *end != '\0')
-        retval = soup_tld_domain_is_public_suffix (end);
-    }
-  }
-
-  g_free (host);
-
-  return retval;
-}
-
-/**
- * ephy_web_view_normalize_or_autosearch_url:
- * @view: an %EphyWebView
- * @url: a URI
- * 
- * Returns a normalized representation of @url, or an autosearch
- * string for it when necessary.
- * 
- * Returns: the normalized @url or autosearch string.
- **/
-char*
-ephy_web_view_normalize_or_autosearch_url (EphyWebView *view, const char *url)
-{
-  char *effective_url;
-  char *scheme;
-  EphyWebViewPrivate *priv = view->priv;
-
-  g_return_val_if_fail (EPHY_IS_WEB_VIEW (view), NULL);
-  g_return_val_if_fail (url, NULL);
-
-  scheme = g_uri_parse_scheme (url);
-
-  /* If the string doesn't look like an URI, let's search it; */
-  if (!ephy_embed_utils_address_has_web_scheme (url) &&
-      scheme == NULL &&
-      !ephy_embed_utils_address_is_existing_absolute_filename (url) &&
-      priv->non_search_regex &&
-      !g_regex_match (priv->non_search_regex, url, 0, NULL) &&
-      !is_public_domain (view, url)) {
-    char *query_param, *url_search;
-
-    url_search = g_settings_get_string (EPHY_SETTINGS_MAIN,
-                                        EPHY_PREFS_KEYWORD_SEARCH_URL);
-
-    if (url_search == NULL || url_search[0] == '\0') {
-      g_free (url_search);
-
-      url_search = g_strdup (_("http://duckduckgo.com/?q=%s&amp;t=epiphany"));
-    }
-
-    query_param = soup_form_encode ("q", url, NULL);
-    /* + 2 here is getting rid of 'q=' */
-    effective_url = g_strdup_printf (url_search, query_param + 2);
-    g_free (query_param);
-    g_free (url_search);
-  } else
-    effective_url = ephy_embed_utils_normalize_address (url);
-
-  if (scheme)
-    g_free (scheme);
-
-  return effective_url;
-}
-
 /**
  * ephy_web_view_load_request:
  * @view: the #EphyWebView in which to load the request
@@ -2145,7 +2058,7 @@ ephy_web_view_load_request (EphyWebView *view,
   g_return_if_fail (WEBKIT_IS_URI_REQUEST (request));
 
   url = webkit_uri_request_get_uri (request);
-  effective_url = ephy_web_view_normalize_or_autosearch_url (view, url);
+  effective_url = ephy_embed_utils_normalize_address (url);
 
   webkit_uri_request_set_uri (request, effective_url);
   g_free (effective_url);
@@ -2211,47 +2124,8 @@ ephy_web_view_load_url (EphyWebView *view,
   g_return_if_fail (EPHY_IS_WEB_VIEW (view));
   g_return_if_fail (url);
 
-  effective_url = ephy_web_view_normalize_or_autosearch_url (view, url);
-
-  /* After normalization there are still some cases that are
-   * impossible to tell apart. One example is <URI>:<PORT> and <NON
-   * WEB SCHEME>:<DATA>. To fix this, let's do a HEAD request to the
-   * effective URI prefxed with http://; if we get OK Status the URI
-   * exists, and we'll go ahead, otherwise we'll try to launch a
-   * proper handler through gtk_show_uri. We only do this in
-   * ephy_web_view_load_url, since this case is only relevant for URIs
-   * typed in the location entry, which uses this method to do the
-   * load. */
-  if (!ephy_embed_utils_address_has_web_scheme (effective_url)) {
-#if 0
-    /* TODO: WebKit2, Network features */
-    SoupMessage *message;
-    SoupSession *session;
-    char *temp_url;
-    HEADAttemptData *data;
-
-    temp_url = g_strconcat ("http://", effective_url, NULL);
-
-    session = webkit_get_default_session ();
-    message = soup_message_new (SOUP_METHOD_HEAD,
-                                temp_url);
-
-    if (message) {
-      data = g_slice_new (HEADAttemptData);
-      data->view = view;
-      data->original_uri = g_strdup (effective_url);
-      soup_session_queue_message (session, message,
-                                  effective_url_head_cb, data);
-    } else {
-      /* If we cannot even create a message fallback to the effective
-       * url, the gtk_show_uri code will make another attempt in
-       * EphyWindow's policy code. */
-      webkit_web_view_load_uri (WEBKIT_WEB_VIEW (view), effective_url);
-    }
-
-    g_free (temp_url);
-#endif
-  } else if (g_str_has_prefix (effective_url, "javascript:")) {
+  effective_url = ephy_embed_utils_normalize_address (url);
+  if (g_str_has_prefix (effective_url, "javascript:")) {
     char *decoded_url;
 
     decoded_url = soup_uri_decode (effective_url);
