@@ -59,17 +59,12 @@ typedef enum
 
 typedef enum
 {
-  EPHY_PREFS_STATE_HISTORY_DATE_FILTER_LAST_HALF_HOUR,
-  EPHY_PREFS_STATE_HISTORY_DATE_FILTER_TODAY,
-  EPHY_PREFS_STATE_HISTORY_DATE_FILTER_LAST_TWO_DAYS,
-  EPHY_PREFS_STATE_HISTORY_DATE_FILTER_LAST_THREE_DAYS,
-  EPHY_PREFS_STATE_HISTORY_DATE_FILTER_LAST_TEN_DAYS,
-  EPHY_PREFS_STATE_HISTORY_DATE_FILTER_EVER,
-} EphyPrefsStateHistoryDateFilter;
+  EPHY_PREFS_PROCESS_MODEL_SHARED_SECONDARY_PROCESS,
+  EPHY_PREFS_PROCESS_MODEL_ONE_SECONDARY_PROCESS_PER_WEB_VIEW
+} EphyPrefsProcessModel;
 
 #define EPHY_PREFS_UI_SCHEMA                     "org.gnome.Epiphany.ui"
 #define EPHY_PREFS_UI_ALWAYS_SHOW_TABS_BAR       "always-show-tabs-bar"
-#define EPHY_PREFS_UI_SHOW_TOOLBARS              "show-toolbars"
 #define EPHY_PREFS_UI_TOOLBAR_STYLE              "toolbar-style"
 #define EPHY_PREFS_UI_DOWNLOADS_HIDDEN           "downloads-hidden"
 #define EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY "tabs-bar-visibility-policy"
@@ -83,10 +78,6 @@ typedef enum
 #define EPHY_PREFS_STATE_RECENT_ENCODINGS       "recent-encodings"
 #define EPHY_PREFS_STATE_BOOKMARKS_VIEW_TITLE   "bookmarks-view-title"
 #define EPHY_PREFS_STATE_BOOKMARKS_VIEW_ADDRESS "bookmarks-view-address"
-#define EPHY_PREFS_STATE_HISTORY_VIEW_TITLE     "history-view-title"
-#define EPHY_PREFS_STATE_HISTORY_VIEW_ADDRESS   "history-view-address"
-#define EPHY_PREFS_STATE_HISTORY_VIEW_DATE      "history-view-date"
-#define EPHY_PREFS_STATE_HISTORY_DATE_FILTER    "history-date-filter"
 
 #define EPHY_PREFS_WEB_SCHEMA                "org.gnome.Epiphany.web"
 #define EPHY_PREFS_WEB_FONT_MIN_SIZE         "min-font-size"
@@ -123,6 +114,8 @@ typedef enum
 #define EPHY_PREFS_ENABLE_CARET_BROWSING          "enable-caret-browsing"
 #define EPHY_PREFS_INTERNAL_VIEW_SOURCE           "internal-view-source"
 #define EPHY_PREFS_RESTORE_SESSION_POLICY         "restore-session-policy"
+#define EPHY_PREFS_RESTORE_SESSION_DELAYING_LOADS "restore-session-delaying-loads"
+#define EPHY_PREFS_PROCESS_MODEL                  "process-model"
 
 #define EPHY_PREFS_LOCKDOWN_SCHEMA            "org.gnome.Epiphany.lockdown"
 #define EPHY_PREFS_LOCKDOWN_FULLSCREEN        "disable-fullscreen"
