@@ -31,9 +31,13 @@
 
 #define EPHY_SNAPSHOT_SERVICE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), EPHY_TYPE_SNAPSHOT_SERVICE, EphySnapshotServicePrivate))
 
+/* Update snapshots after one week. */
+#define SNAPSHOT_UPDATE_THRESHOLD (60 * 60 * 24 * 7)
+
 struct _EphySnapshotServicePrivate
 {
   GnomeDesktopThumbnailFactory *factory;
+  GHashTable *cache;
 };
 
 G_DEFINE_TYPE (EphySnapshotService, ephy_snapshot_service, G_TYPE_OBJECT)
@@ -50,9 +54,11 @@ ephy_snapshot_service_class_init (EphySnapshotServiceClass *klass)
 static void
 ephy_snapshot_service_init (EphySnapshotService *self)
 {
-
   self->priv = EPHY_SNAPSHOT_SERVICE_GET_PRIVATE (self);
   self->priv->factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
+  self->priv->cache = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                             (GDestroyNotify)g_free,
+                                             (GDestroyNotify)g_free);
 }
 
 typedef struct {
@@ -105,6 +111,8 @@ get_snapshot_for_url_thread (GSimpleAsyncResult *result,
     return;
   }
 
+  g_hash_table_insert (service->priv->cache, g_strdup (data->url), g_strdup (data->path));
+
   data->snapshot = gdk_pixbuf_new_from_file (data->path, &error);
   if (data->snapshot == NULL) {
     g_simple_async_result_set_error (result,
@@ -151,6 +159,68 @@ snapshot_async_data_free (SnapshotAsyncData *data)
   g_slice_free (SnapshotAsyncData, data);
 }
 
+static GdkPixbuf *
+ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface,
+                                        cairo_surface_t *favicon)
+{
+  GdkPixbuf *snapshot, *scaled;
+  int orig_width, orig_height;
+  float orig_aspect_ratio, dest_aspect_ratio;
+  int x_offset, new_width = 0, new_height;
+
+  orig_width = cairo_image_surface_get_width (surface);
+  orig_height = cairo_image_surface_get_height (surface);
+
+  if (orig_width < EPHY_THUMBNAIL_WIDTH ||
+      orig_height < EPHY_THUMBNAIL_HEIGHT) {
+    snapshot = gdk_pixbuf_get_from_surface (surface,
+                                            0, 0,
+                                            orig_width, orig_height);
+    scaled = gdk_pixbuf_scale_simple (snapshot,
+                                      EPHY_THUMBNAIL_WIDTH,
+                                      EPHY_THUMBNAIL_HEIGHT,
+                                      GDK_INTERP_TILES);
+  } else {
+    orig_aspect_ratio = orig_width / (float)orig_height;
+    dest_aspect_ratio = EPHY_THUMBNAIL_WIDTH / (float)EPHY_THUMBNAIL_HEIGHT;
+
+    if (orig_aspect_ratio > dest_aspect_ratio) {
+      /* Wider than taller, crop the sides. */
+      new_width = orig_height * dest_aspect_ratio;
+      new_height = orig_height;
+      x_offset = (orig_width - new_width) / 2;
+    } else {
+      /* Crop the bottom otherwise. */
+      new_width = orig_width;
+      new_height = orig_width / (float)dest_aspect_ratio;
+      x_offset = 0;
+    }
+
+    snapshot = gdk_pixbuf_get_from_surface (surface, x_offset, 0, new_width, new_height);
+    scaled = gnome_desktop_thumbnail_scale_down_pixbuf (snapshot,
+                                                        EPHY_THUMBNAIL_WIDTH,
+                                                        EPHY_THUMBNAIL_HEIGHT);
+  }
+
+  g_object_unref (snapshot);
+
+  if (favicon) {
+    GdkPixbuf* fav_pixbuf;
+    int favicon_size = 16;
+    int x_offset = 6;
+    int y_offset = gdk_pixbuf_get_height (scaled) - favicon_size - x_offset;
+
+    fav_pixbuf = ephy_pixbuf_get_from_surface_scaled (favicon, favicon_size, favicon_size);
+    gdk_pixbuf_composite (fav_pixbuf, scaled,
+                          x_offset, y_offset, favicon_size, favicon_size,
+                          x_offset, y_offset, 1, 1,
+                          GDK_INTERP_NEAREST, 255);
+    g_object_unref (fav_pixbuf);
+  }
+
+  return scaled;
+}
+
 static void
 snapshot_saved (EphySnapshotService *service,
                 GAsyncResult *result,
@@ -317,6 +387,7 @@ save_snapshot_thread (GSimpleAsyncResult *result,
                                                   data->url,
                                                   data->mtime);
   data->path = gnome_desktop_thumbnail_path_for_uri (data->url, GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
+  g_hash_table_insert (service->priv->cache, g_strdup (data->url), g_strdup (data->path));
 }
 
 GQuark
@@ -464,6 +535,7 @@ ephy_snapshot_service_get_snapshot_async (EphySnapshotService *service,
 {
   GSimpleAsyncResult *result;
   const char *uri;
+  time_t current_time = time (NULL);
 
   g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
   g_return_if_fail (WEBKIT_IS_WEB_VIEW (web_view));
@@ -477,7 +549,7 @@ ephy_snapshot_service_get_snapshot_async (EphySnapshotService *service,
 
   /* Try to get the snapshot from the cache first if we have a URL */
   uri = webkit_web_view_get_uri (web_view);
-  if (uri)
+  if (uri && current_time - mtime <= SNAPSHOT_UPDATE_THRESHOLD)
     ephy_snapshot_service_get_snapshot_for_url_async (service,
                                                       uri, mtime, cancellable,
                                                       (GAsyncReadyCallback)got_snapshot_for_url,
@@ -578,63 +650,172 @@ ephy_snapshot_service_save_snapshot_finish (EphySnapshotService *service,
   return retval;
 }
 
-GdkPixbuf *
-ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface,
-                                        cairo_surface_t *favicon)
+const char *
+ephy_snapshot_service_lookup_snapshot_path (EphySnapshotService *service,
+                                            const char *url)
 {
-  GdkPixbuf *snapshot, *scaled;
-  int orig_width, orig_height;
-  float orig_aspect_ratio, dest_aspect_ratio;
-  int x_offset, new_width = 0, new_height;
+  g_return_val_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service), NULL);
 
-  orig_width = cairo_image_surface_get_width (surface);
-  orig_height = cairo_image_surface_get_height (surface);
-
-  if (orig_width < EPHY_THUMBNAIL_WIDTH ||
-      orig_height < EPHY_THUMBNAIL_HEIGHT) {
-    snapshot = gdk_pixbuf_get_from_surface (surface,
-                                            0, 0,
-                                            orig_width, orig_height);
-    scaled = gdk_pixbuf_scale_simple (snapshot,
-                                      EPHY_THUMBNAIL_WIDTH,
-                                      EPHY_THUMBNAIL_HEIGHT,
-                                      GDK_INTERP_TILES);
+  return g_hash_table_lookup (service->priv->cache, url);
+}
+
+static void
+get_snapshot_path_for_url_thread (GSimpleAsyncResult *result,
+                                  EphySnapshotService *service,
+                                  GCancellable *cancellable)
+{
+  SnapshotForURLAsyncData *data;
+
+  data = (SnapshotForURLAsyncData *)g_simple_async_result_get_op_res_gpointer (result);
+  data->path = gnome_desktop_thumbnail_factory_lookup (service->priv->factory, data->url, data->mtime);
+  if (!data->path) {
+    g_simple_async_result_set_error (result,
+                                     EPHY_SNAPSHOT_SERVICE_ERROR,
+                                     EPHY_SNAPSHOT_SERVICE_ERROR_NOT_FOUND,
+                                     "Snapshot for url \"%s\" not found in cache", data->url);
+  } else {
+    g_hash_table_insert (service->priv->cache, g_strdup (data->url), g_strdup (data->path));
+  }
+}
+
+void
+ephy_snapshot_service_get_snapshot_path_for_url_async (EphySnapshotService *service,
+                                                       const char *url,
+                                                       const time_t mtime,
+                                                       GCancellable *cancellable,
+                                                       GAsyncReadyCallback callback,
+                                                       gpointer user_data)
+{
+  GSimpleAsyncResult *result;
+  SnapshotForURLAsyncData *data;
+  const char *path;
+
+  g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
+  g_return_if_fail (url != NULL);
+
+  result = g_simple_async_result_new (G_OBJECT (service), callback, user_data,
+                                      ephy_snapshot_service_get_snapshot_path_for_url_async);
+
+  data = snapshot_for_url_async_data_new (url, mtime);
+  g_simple_async_result_set_op_res_gpointer (result, data, (GDestroyNotify)snapshot_for_url_async_data_free);
+
+  path = g_hash_table_lookup (service->priv->cache, url);
+  if (path) {
+    data->path = g_strdup (path);
+    g_simple_async_result_complete_in_idle (result);
+  } else {
+    g_simple_async_result_run_in_thread (result,
+                                         (GSimpleAsyncThreadFunc)get_snapshot_path_for_url_thread,
+                                         G_PRIORITY_LOW, cancellable);
+  }
+  g_object_unref (result);
+}
+
+char *
+ephy_snapshot_service_get_snapshot_path_for_url_finish (EphySnapshotService *service,
+                                                        GAsyncResult *result,
+                                                        GError **error)
+{
+  SnapshotForURLAsyncData *data;
+  char *retval;
+
+  g_return_val_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service), NULL);
+  g_return_val_if_fail (g_simple_async_result_is_valid (result,
+                                                        G_OBJECT (service),
+                                                        ephy_snapshot_service_get_snapshot_path_for_url_async),
+                        NULL);
+
+  if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+    return NULL;
+
+  data = (SnapshotForURLAsyncData *)g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result));
+
+  retval = data->path;
+  data->path = NULL;
+
+  return retval;
+}
+
+static void
+got_snapshot_path_for_url (EphySnapshotService *service,
+                           GAsyncResult *result,
+                           GSimpleAsyncResult *simple)
+{
+  SnapshotAsyncData *data;
+
+  data = (SnapshotAsyncData *)g_simple_async_result_get_op_res_gpointer (simple);
+  data->path = ephy_snapshot_service_get_snapshot_path_for_url_finish (service, result, NULL);
+  if (data->path) {
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
   } else {
-    orig_aspect_ratio = orig_width / (float)orig_height;
-    dest_aspect_ratio = EPHY_THUMBNAIL_WIDTH / (float)EPHY_THUMBNAIL_HEIGHT;
-
-    if (orig_aspect_ratio > dest_aspect_ratio) {
-      /* Wider than taller, crop the sides. */
-      new_width = orig_height * dest_aspect_ratio;
-      new_height = orig_height;
-      x_offset = (orig_width - new_width) / 2;
+    ephy_snapshot_service_take_from_webview (simple);
+  }
+}
+
+void
+ephy_snapshot_service_get_snapshot_path_async (EphySnapshotService *service,
+                                               WebKitWebView *web_view,
+                                               const time_t mtime,
+                                               GCancellable *cancellable,
+                                               GAsyncReadyCallback callback,
+                                               gpointer user_data)
+{
+  GSimpleAsyncResult *result;
+  SnapshotAsyncData *data;
+  const char *uri;
+  time_t current_time = time (NULL);
+
+  g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
+  g_return_if_fail (WEBKIT_IS_WEB_VIEW (web_view));
+
+  result = g_simple_async_result_new (G_OBJECT (service), callback, user_data,
+                                      ephy_snapshot_service_get_snapshot_path_async);
+
+  data = snapshot_async_data_new (web_view, mtime, cancellable);
+  g_simple_async_result_set_op_res_gpointer (result, data, (GDestroyNotify)snapshot_async_data_free);
+
+  uri = webkit_web_view_get_uri (web_view);
+  if (uri && current_time - mtime <= SNAPSHOT_UPDATE_THRESHOLD) {
+    const char *path = g_hash_table_lookup (service->priv->cache, uri);
+
+    if (path) {
+      data->path = g_strdup (path);
+      g_simple_async_result_complete_in_idle (result);
     } else {
-      /* Crop the bottom otherwise. */
-      new_width = orig_width;
-      new_height = orig_width / (float)dest_aspect_ratio;
-      x_offset = 0;
+      ephy_snapshot_service_get_snapshot_path_for_url_async (service,
+                                                             uri, mtime, cancellable,
+                                                             (GAsyncReadyCallback)got_snapshot_path_for_url,
+                                                             g_object_ref (result));
     }
+  } else {
+    g_idle_add ((GSourceFunc)ephy_snapshot_service_take_from_webview, g_object_ref (result));
+  }
+
+  g_object_unref (result);
+}
+
+char *
+ephy_snapshot_service_get_snapshot_path_finish (EphySnapshotService *service,
+                                                GAsyncResult *result,
+                                                GError **error)
+{
+  SnapshotAsyncData *data;
+  char *retval;
 
-    snapshot = gdk_pixbuf_get_from_surface (surface, x_offset, 0, new_width, new_height);
-    scaled = gnome_desktop_thumbnail_scale_down_pixbuf (snapshot,
-                                                        EPHY_THUMBNAIL_WIDTH,
-                                                        EPHY_THUMBNAIL_HEIGHT);
-  }
+  g_return_val_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service), FALSE);
+  g_return_val_if_fail (g_simple_async_result_is_valid (result,
+                                                        G_OBJECT (service),
+                                                        ephy_snapshot_service_get_snapshot_path_async),
+                        FALSE);
+
+  if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+    return NULL;
+
+  data = (SnapshotAsyncData *)g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result));
 
-  g_object_unref (snapshot);
-
-  if (favicon) {
-    GdkPixbuf* fav_pixbuf;
-    int favicon_size = 16;
-    int x_offset = 6;
-    int y_offset = gdk_pixbuf_get_height (scaled) - favicon_size - x_offset;
-    fav_pixbuf = ephy_pixbuf_get_from_surface_scaled (favicon, favicon_size, favicon_size);
-    gdk_pixbuf_composite (fav_pixbuf, scaled,
-                          x_offset, y_offset, favicon_size, favicon_size,
-                          x_offset, y_offset, 1, 1,
-                          GDK_INTERP_NEAREST, 255);
-    g_object_unref (fav_pixbuf);
-  }
+  retval = data->path;
+  data->path = NULL;
 
-  return scaled;
+  return retval;
 }
